#!/usr/bin/perl -w
#
# Copyright (C) 1998, 1999 VMware, Inc.  All Rights Reserved.
#
# Host and site-specific installation for vmware.
#
# If your copy of perl is not in /usr/bin, please adjust the line above.

use strict;

my ($bindir,  $legacylibdir, $mandir, $initdir);
my ($default_bindir,  $default_mandir, 
    $default_initdir, $modulesdir, $vmware_tmpdir,
    $vmware_etc, $vmware_locations, $vmnet1, %previousAnswer);


my $defaultroot = "/usr/local";

$default_bindir = defined($ENV{BINDIR})? $ENV{BINDIR} : "$defaultroot/bin";
$default_mandir = defined($ENV{MANDIR})? $ENV{MANDIR} : "$defaultroot/man";
$default_initdir = defined($ENV{INITDIR})? $ENV{INITDIR} : "/etc/rc.d";
$modulesdir = "/lib/modules";
$vmware_tmpdir = "/tmp/vmware-tmp";
$vmware_etc = "/etc/vmware";
$vmware_locations = "$vmware_etc/locations";
$vmnet1 = "vmnet1";			# host-only networking interface


my $supportEMail = 'support@vmware.com';
my $createdBuildDir = "";

my ($umask, $version, $smp);

my @programs = ("vmware", "vmware-wizard","vmnet-bridge", "vmnet-sniffer", "vmnet-dhcpd");

my @libfiles = ("nvram");
my $driver;
my $option;
my $vnet;

my $verbose = 0;

####
####  Don't you just hate having to read these damn things?  We do too,
#### that's why we've put this in.  And we'll even force you to read it.
####
sub Show_EULA {
   print << "EOMARKETING";

You must read and accept the End User License Agreement to continue.
Press return to display it.
EOMARKETING

# The above message used to say:
#
#   You must read and accept the End User License Agreement to continue.
#   Well, you must at least accept it.  Press return to display it.
#
# which is very cute, but I had to change it because we don't want
# to give the impression we're not taking ourselves seriously.  Well,
# at least not before the IPO.

   <STDIN>;
   system((defined($ENV{PAGER}) ? $ENV{PAGER} : "/bin/more") . " EULA");
   REPLY: while (1) {
       # Don't use standard query here, because people may
       # be using <return> to scroll through the EULA.
       my $reply;
       print "Do you accept? (yes/no) ";
       chop($reply = <STDIN>);
       last REPLY if $reply =~ /^[Yy]$|^[Yy][Ee][Ss]$/;
       die "\n### The installation is aborted.  Please try again\n" .
	   "### when you are ready to accept.\n"
	       if $reply =~ /^[Nn]$|^[Nn][Oo]$/;
       print "Please reply yes or no.\n";
   }
   print "Thank you.\n\n";
}

sub message {
  my $msg = shift;
  if ($verbose) { 
    print $msg;
  }
}

sub query {

    my ($message, $defaultreply) = @_;
    my $reply;
    my $optSpace = substr($message, -1) eq "\n" ? "" : " ";
    if ($defaultreply ne "") {
	print $message . $optSpace . "[" . $defaultreply . "] ";
    } else {
	print $message . $optSpace;
    }
	   
    chop($reply = <STDIN>);
    $reply =~ s/^\s*//;			# Kill leading whitespace
    $reply =~ s/\s*$//;			# Kill trailing whitespace
    $reply = $defaultreply if $reply eq "";
    return $reply;
}

sub makeDirectory {

    my $path = $_[0];
    my $slashpos;
    my $head;
    my $tail;

    if ("$path" eq "") {
	return;
    }

    if (not -d "$path") {
	if (-e "$path") {
	    die "\n### $path exists but it is not a directory.\n";
	} else {
	    $slashpos = rindex($path, "/");
	    $head = substr($path, 0, $slashpos);
	    makeDirectory($head);
            Log("directory $path\n");
	    mkdir($path, 0755) or die "\n### Could not create $path: $!.\n";
	}
    }
}


sub Log { 
  my $string = shift;
  print LOCATIONS $string;
}

sub OpenLog {
  my $buildNr = q"$Name: build-101 $";
  
  if (!(-d $vmware_etc)) { 
    mkdir($vmware_etc,0755) or die "Could not create $vmware_etc\n";
  }
  open(LOCATIONS,">$vmware_locations") or die "Could not open $vmware_locations\n";

  #### page 110 of the book

  select((select(LOCATIONS), $| = 1)[0]);

  Log("directory $vmware_etc\n");
  Log("VERSION  $buildNr \n");
}

sub checkDir {

    my ($dir, $option) = @_;
    my $answer;

    message("checkDir $dir $option \n");
    if ("$dir" !~ /^\//) {
	chop($answer = `pwd`);
	$dir = $answer . "/" . $dir;
    }

    if (not -d "$dir") {
	if ($option eq "uninstall") {
	    warn "\n### $dir is not a valid directory.\n";
	} else {
	    die "\n### $dir exists and is not a directory.\n" if -e "$dir";

	    if ($option eq "default" ||
		query("Directory $dir does not exist.\n" .
		      "Create it (including needed parent directories)?",
		      "yes") =~ /^[Yy]/) {
		message("Making directory $dir\n");
		makeDirectory($dir);
	    } else {
              return 0;
	    }
	}
    }
    return 1;
}

# Set bindir, mandir, libdir and initdir.
# Also check as early as possible if uninstallation is
# required.


sub Init {

  my $arg = $_[0];
  my $operation;

  my $diroption = $arg =~ /uninstall$/ ? "uninstall" :
      $arg eq "default" ? "default" : $arg;

  if ($arg eq "legacy") {
    $diroption = "uninstall";
    $operation = "legacy uninstallation";
  } elsif ($arg eq "uninstall") { 
    $operation = "uninstallation";
  } else {
    $operation = "installation";
  }


  ###
  ### create a misc directory if not present
  ###
  
  if (-d "$modulesdir/preferred") { 
    checkDir("$modulesdir/preferred/misc",$diroption);
  } else { 
    checkDir("$modulesdir/$version/misc",$diroption);
    }
  
  
  if ((! defined($ENV{BINDIR}) ||
       ! defined($ENV{MANDIR}) ||
       ! defined($ENV{INITDIR}))
      && $arg eq "host") {
    print << "EOM";
    
    The host installation requires the knowledge of the site
    installation directories, even though no files will be
    installed in those directories.

EOM
  }

  while (1) { 
    $bindir = defined($ENV{BINDIR}) ? $ENV{BINDIR} :
      query("Enter binaries $operation directory:",
	    "$defaultroot/bin");
    
    
    if (checkDir($bindir, $diroption)) { 
      last;
    } else {
      if (defined($ENV{BINDIR})) { 
        undef $ENV{BINDIR};
      }
    }
  }
  
  if ($arg ne "legacy") { 
    Log("answer BINDIR $bindir\n");
  }

  if ($arg eq "all") { 
    print << "EOM";
Binaries installation directory is $bindir.
This is the location of the executables.
EOM
  }

  $defaultroot = substr($bindir, 0, rindex($bindir, "/"));
  
  if ($arg eq "legacy") { 
    while (1) {
      $legacylibdir = defined($ENV{LIBDIR}) ? $ENV{LIBDIR} :
        query("Enter library $operation directory:",
              "$defaultroot/lib/vmx86");
      
      if (checkDir($legacylibdir, $diroption)) { 
        last;
      } else {
        if (defined($ENV{LIBDIR})) { 
          undef $ENV{LIBDIR};
        }
      }
    } 
  }

  while (1) { 
    $mandir = defined($ENV{MANDIR}) ? $ENV{MANDIR} :
      query("Enter $operation directory for manual pages:",
            "$defaultroot/man");
    
    message("user entered $mandir\n");
    if (checkDir("$mandir/man1", $diroption)) {
      last;
    } else {
      if (defined($ENV{MANDIR})) { 
        undef $ENV{MANDIR};
      }
    }
  }

  if ($arg ne "legacy") { 
    Log("answer MANDIR $mandir\n");
  }
  
  
  if ($arg eq "all") {
    print << "EOM";
    Manual installation directory is $mandir.
    This is the location of the manual pages.
EOM
  }


  while (1) { 
    
    $initdir = defined($ENV{INITDIR}) ? $ENV{INITDIR} :
      query(
            "Enter the top directory under which the init scripts reside.  It should contain\n" .
            "init.d/, rc3.d/, rc5.d/, and rc6.d/:", "$default_initdir");
    
    
    if (checkDir("$initdir/init.d", $diroption) && 
        checkDir("$initdir/rc3.d", $diroption) &&
        checkDir("$initdir/rc5.d", $diroption) &&
        checkDir("$initdir/rc6.d", $diroption)) { 
      last;
    } 
  }  
  
  if ($arg ne "legacy") { 
    Log("answer INITDIR $initdir\n");
  }
  
  print << "EOM";
The boot-time init script will be stored under $initdir.
This is the location of the script that loads the vmware
kernel modules on reboot.
EOM


}


sub Install {
  my ($mode, $src, $dst) = @_;

  open(SRC, "<$src") or die "Open: $src: $!.\n";
  open(DST, ">$dst") or die "Open: $dst: $!.\n";
  chmod $mode, $dst or die "chmod $dst: $!.\n";

  if ($src =~ /\.dist$/) {		# Need to make changes to contents
    while(defined($_ = <SRC>)) {
      s/%BINDIR%/$bindir/g;
      s/%INITDIR%/$initdir/g;
      s/%ETCDIR%/$vmware_etc/g;
#      s/%DRIVER%/$driver/g;
#      s/%VNET%/$vnet/g;
       
      print DST $_;
    }
  } else {				# Just copy
    while(defined($_ = <SRC>)) {
      print DST $_;
    }
  }

  close(SRC);
  close(DST);

  Log("file $dst\n");
  
}





sub checkHostPlatform {
  my($sysname, $release, @r, $answer, $threecom, $grep);
  
  chop($sysname = `uname -s`);
  
  if ($sysname ne "Linux") {
    die "You are not running Linux.\n" .
      "This version of the product is for Linux only.";
  }
  
  chop($release = `uname -r`);
  @r = split(/\./, $release);
  
  if ($r[0] < 2) {
    die "You are running Linux release $release.\n" .
      "This product only runs on 2.0 and later kernels.\n";
  }

  $grep = 0;
  $grep = "/bin/egrep" if -e "/bin/egrep";
  $grep = "/usr/bin/egrep" if -e "/usr/bin/egrep";
  $threecom = ! (-e("/proc/ioports")) || ! $grep
    || (`$grep -i '(3c90|3c59)' /proc/ioports` ne "");
  
  if ($threecom && ($r[1] == 0) && ($r[2] == 34 || $r[2] == 35)) {
    $answer =
	query("The installation script has detected that you are\n" . 
	      "running Linux version $release possibly with a 3Com\n" .
	      "networking card.  Linux kernel versions 2.0.34 and 2.0.35\n" .
	      "have a bug in the 3Com driver that interacts badly with\n".
	      "this product.  Specifically, the real machine will\n".
	      "occasionally hang and will require a hard reset.\n" .
	      "This bug has been fixed in Linux 2.0.36.\n" .
	      "\n" .
	      "Do you want to continue the installation anyway?", "no");
    ($answer =~ /^[yY]/) || exit(0);
  }
}


sub BuildFailure {
  my $driver_name = shift;

  print <<"FATAL_VERSION";

The build of the kernel module $driver_name failed.

Double check that /usr/include/linux and /usr/src/linux both refer to
the source tree that corresponds to the currently running kernel version.

If you detect a mismatch, update these softlinks and retry.

Please follow the following steps if you still have problems.

1. untar 'driver-only.tar' or 'vmnet-only.tar' 
2. cd into that directory
3a. On a UP kernel, type 'make'
3b  On an MP kernel, type 'make SUPPORT_SMP=1'

4. If the compilation is successful, try installing the module:

    /sbin/insmod -o vmmon driver-<kernel version>
              --- or ---   
    /sbin/insmod -o vmnet vmnet-<kernel version>

5. If this step is successful, copy the module into the distribution directory
   as  vmmon-<kernel version> or vmnet-<kernel version>


Please file an incident report at http://www.vmware.com/support if you
are still having problems. 

FATAL_VERSION
    die "Installation failed.\n";
}


###  How brave are you?  This little uber-routine will try and automagically
### build the driver (and load it just to be sure).
sub Build_Driver {
  my ($base, $mod_base,$option) = @_;
  my ($driver_name, $build_dir);

  #  Do this work in /tmp or TMPDIR b/c we need a local filsystem to avoid
  # NFS vs. root permission problems.
  $build_dir = defined($ENV{TMPDIR}) ? $ENV{TMPDIR} : "/tmp";

  die "Cannot untar and build driver in $build_dir, please set \$TMPDIR\n"
     unless -d "$build_dir";
  
  $build_dir = "$build_dir/vmware-tmp";

  if (not -d "$build_dir") {
    $createdBuildDir = $build_dir;
    mkdir "$build_dir", 0700 or 
       die "Unable to create $build_dir: $!\n";
  }

  # Okay, let's see if we have a tarfile?
  if (-r "$base-only.tar") {
    message "Extracting $base source from $base-only.tar\n";
    system("tar", "-C", "$build_dir", "-xf", "$base-only.tar") 
        and warn "Unable to untar!\n";
  }

  # Now let's see if we have source?
  if (! -r "$build_dir/$base-only/Makefile") {
    die << "EOM";

Unable to find $base source.  Either copy $base-only.tar to the current
directory from the distribution or untar it yourself into the current
directory.  If your distribution did not contain $base source, please 
contact http://www.vmware.com for information about obtaining a working 
module.

EOM
  }

  ####
  #### we now support egcs 
  ####
  
  if (0) { 
    # Fail if gcc on this system is the EGCS compiler.
    
    if (`gcc --version | grep -i egcs` ne "") {
      die "The default gcc on this system is EGCS, which is not\n" .
        "compatible with this software.\n";
    }
  }

  # Okay, now let's see how good we are.
  $driver_name = $mod_base . $version;
  if ($smp) {
    message "Building the $base with SMP support\n";
    system("make","-C","$build_dir/$base-only/", "auto-build", "SUPPORT_SMP=1")
      and BuildFailure($driver_name); 
  } else {
    message "Building the $base for a  single processor machine\n";
    system("make", "-C", "$build_dir/$base-only/", "auto-build") and
      BuildFailure($driver_name); 
  }
  
  # Did it work, did it work?
  message "Verifying the build worked\n";
  if (Try_Insmod($driver_name, $build_dir)) {
    print "Build was successful: Driver name is $driver_name\n";
  } else {
    BuildFailure($driver_name);
  }

  return ($driver_name, $build_dir);
}



sub Try_Insmod {
  my ($mod_name, $path) = @_;
  my $retval;

  # Bitter.  I am bitter.  Insmod prints on stderr and so I need sh -c.
  $retval = `sh -c "/sbin/insmod -f $path/$mod_name 2>&1"`;
  if ($? != 0) {
    print $retval;
    if ($retval =~ /Device or resource busy/) {
      die << "EOM";
      
Either you currently have a VM running, or another module is currently
using the device number required by our device driver.  Please make certain
all VM's are exited, and that you have either done an uninstall already or 
are doing an upgrade installation. If you still get this message, please
contact http://www.vmware.com/support.
EOM
    }
    return 0;
  } else {
    system("/sbin/rmmod", "$mod_name") and 
      warn "Able to load $mod_name, but not unload it!\n";
    return 1;
  }
}

###  This routine goes through a distribution directory and tries to find a
### vmx86 driver that it can successfully load, or else it tries to build
### one.  It returns the name of the file that contains a valid driver.  It
### gets passed the base of the module (vnet or driver)


sub Find_Module {
  
  my ($base,$option)   = @_;
  my $mod_base = (($base eq "driver") ? "vmmon":"vmnet") .
    ($smp ? "-smp-" : "-up-");
  my @options = ();
  my ($ideal, $guess, $driver, $driver_path,$envVar);
  
  $driver_path = ".";
  $ideal = $mod_base . $version;
  if (-r $ideal) {
    message "Using ideal driver $ideal\n";
    if (Try_Insmod($ideal, ".")) {
      message "Success!  Using $ideal.\n";
      $driver = $ideal;
    } else {
      warn "Unable to use $ideal, please rebuild it locally\n";
    }
  } else {
    print "No exact match for $ideal.  Checking for working alternatives.\n";
    ($guess = $ideal) =~ s/.$//;
    @options = glob("$guess?");
    if ($#options == -1) {
      warn "No alternatives found, please build a driver locally\n";
    } else {
      message "Identifying a match that works with this kernel " .
        "(ignore any errors)\n";
      foreach $guess (@options) {
        ($driver = $guess and last) if Try_Insmod($guess, ".");
      }
    }
  }
  
  if (defined($driver)) {
    print "Module '$driver' detected to work correctly\n";
  } else {
    print "None of the prebuilt modules seems to work on this machine.\n";
    $envVar = "BUILDR_$base";
    message("ENV $envVar \n");

    if (defined($ENV{$envVar})) {
      $_ = $ENV{$envVar};
    } elsif ($option eq "default") { 
      $_ = "yes";
    } else { 
      $_ = query("I can try and build one for you, would you like me to try?",
                 "yes");
    }
    
    Log("answer $envVar $_\n");
    
    if (/^[Nn]/) {
        die << "EOM";
Not building automatically.  To build by hand, please do the following:
        
tar xf $base-only.tar
cd $base-only
make
              
and then verify that the module in
             
$base-$version/$ideal
                  
will load.
EOM
     } else {
       ($driver, $driver_path) = &Build_Driver($base, $mod_base,$option) unless /^[Nn]/;
     }
  }
  
  return ($driver, $driver_path);
}

sub printFindPingHelp {
    print <<'EOF';

The ping program sends ICMP echo messages to a specific host and
listens for replies.  We use it when probing for an available
subnet for host-only networking to identify networks that are
not reachable from your host machine.  ping is normally installed
on your system in a standard location such as /bin, but it might
be somewhere unexpected in which case you must specify the full
UNIX pathname for the program.  If you do not identify where this
program is located then we cannot automatically locate a network
number to use and you must manually specify how to setup the
host-only network.

EOF
}

sub findPing {
    my $answer;
    my $ping;

    # NB: add alternative locations for ping as needed
    $ping = "/bin/ping" if -e "/bin/ping";
    if ($ping eq "") {
	print "\n";
	print "Could not locate the ping program in the expected locations.\n";
	do {
	    $answer = query("What is the full pathname for the ping program" .
			"(e.g. /bin/ping)?", "");
	    if (!(-e $answer)) {
		print "\nSorry that is not an executable program.\n";
		do {
		    $answer = query("Try again? (yes/no/help)\n", "yes");
		    printFindPingHelp() if $answer =~ /^[Hh]/;
		    return ("") if $answer =~ /^[Nn]/;
		} until $answer =~ /^[Yy]/;
	    } else {
		$ping = $answer;
	    }
	} until $ping ne "";
    }
    return ($ping);
}

#
# Routine to generate an interfaces specification for smb.conf
# that lists each network interface on a Linux host that is
# marked "up".  We assume that ifconfig w/o any arguments will
# list all such interfaces.  We also assume a bunch about the
# (convoluted) printing format of ifconfig.
#
sub printSmbInterfaces {
    my @fields;
    my $_if = "";
    my $nif = 0;
    my (@ifs, %addrs, %masks);

    open(IFCONFIG, "/sbin/ifconfig|");
    while (<IFCONFIG>) {
	if (/^[a-zA-Z]/) {
	    @fields = split(/[ ]+/);
	    $_if = $fields[0];
	    $ifs[$nif++] = $_if;
	} elsif (/inet addr/) {
	    chop;
	    ($addrs{$_if} = $_) =~ s/.*[iI]net [aA]ddr://;
	    $addrs{$_if} =~ s/ .*//;
	    ($masks{$_if} = $_) =~ s/.*[mM]ask://;
	    $masks{$_if} =~ s/ .*//;
	}
    }

    my $s = "";
    my $sep = "";
    foreach $_if (@ifs) {
	if ($_if ne "" && $_if ne "lo") {
	    my $mask = $masks{$_if};
	    my @parts = split(/[.]/, $mask);

	    my %map = (
		255, 8,
		254, 7,
		252, 6,
		248, 5,
		240, 4,
		224, 3,
		192, 2,
		128, 1,
		0,   0,
	    );
	    my $v;
	    my $bits = 0;

	    foreach $v (@parts) {
		$bits += $map{int($v)};
	    }
	    $s .= sprintf("%s%s/%d", $sep, $addrs{$_if}, $bits);
	    $sep = " ";
	}
    }
    return $s;
}

sub probeForSubnet {
    my $subnet = shift;

    my $dot;
    my $probeAddress;
    my $status;

    my $ping = findPing();
    if ($ping ne "") {
	for ($dot = "0"; $dot <= "255"; $dot++) {
	    # by convention HOS is <subnet>.1
	    $probeAddress = $subnet . "." . $dot . ".1";
	    message("    ping $probeAddress...");
	    #
	    # ping exits with:
	    #   1 if no replys
	    #   2 if an error
	    #   0 otherwise
	    #
	    $status = (system("$ping -n -c 2 $probeAddress >/dev/null 2>&1") >> 8);
	    if ($status == 1) {
		message("(no response)\n");
		last;
	    }
	    message("(answered)\n");
	}
    } else {
	$dot = "256";			# i.e. nothing located
    }
    return ($subnet . "." . $dot);
}

sub probeForNetConfig {
    my $net;
    my $subnet;
    my $netmask = "255.255.255.0";	# XXX calculate dynamically

    message("\nProbing for an unused private network number:\n");
    # first try the private Class C networks
    foreach $net ("192.168", "172.16") {
	$subnet = probeForSubnet($net);
	if ($subnet ne $net . ".256") {
	    print "Network number $subnet (netmask $netmask) appears unused.\n";
	    # XXX ask user if selection is ok
	    return ($subnet, $netmask);
	}
    }
    # XXX can't get a here document to work right...
    print <<'EOF';

We were unable to locate an unused Class C subnet in the range
of private network numbers.  For each subnet that we tried we
received a response to our ICMP ping packets from a host at the
network address intended for assignment to this host machine.
Because no private subnet appears to be unused you will need to
explicitly specify a network number.

EOF
    return ("", "");
}

sub promptForNetConfig {
    my $net = "";
    my $netmask = "";
    my $answer;

    do {
	do {
	    $answer = query("Network number (use a.b.c format)?", $net);
	    # XXX validate input
	} until $answer ne "";
	$net = $answer;
	$answer = "";
	# calculate netmask from network number?
	do {
	    $answer = query("Network mask (use a.b.c.d format)?", $netmask);
	    # XXX validate input
	} until $answer ne "";
	$netmask = $answer;

	$answer = query("Use network $net and mask $netmask? (yes/no)", "yes");
    } until $answer =~ /^[Yy]/;
    return ($net, $netmask);
}

sub writeDHCPConfigFile {
    my $net = shift;
    my $netmask = shift;

    # XXX deal with open errors
    open(DHCPCONFIG, ">$vmware_etc/$vmnet1.conf");
    # XXX this would be a here document but perl keeps barfing on it
    my $handle = select(DHCPCONFIG);
    print <<"EOF";
#
# Configuration file for ISC 2.0b6pl1 vmnet-dhcpd operating on $vmnet1.
#
# Beware, this file was generated by the VMware installation
# script.  If you modify it, it will be clobbered the next time
# you run the installation script (TO BE FIXED).
#
allow unknown-clients;
default-lease-time 1800;		# 30 minutes
max-lease-time 7200;			# 2 hours

subnet $net.0 netmask $netmask {
    range $net.128 $net.255;		# up to 127 VM's
    option domain-name-servers $net.1;	# by convention HOS is .1
    option broadcast-address $net.255;
}
EOF
    select($handle);
    close(DHCPCONFIG);

    Log("answer VNET_HOSTONLY_NETWORK $net\n");
    Log("answer VNET_HOSTONLY_NETMASK $netmask\n");
    Log("file $vmware_etc/$vmnet1.conf\n");
}


sub printHostOnlyHelp {
    print <<'EOF';

Virtual machines configured to use host-only networking are placed on
a virtual network that is confined to this host.  Virtual machines on
this network can communicate with each other and the host, but noone else.

To setup this host-only networking you need to select a network number
that is unreachable from the host.  We can automatically select this
number for you, or you can specify a network number that you want.

The automatic selection process works by testing a series of Class C
subnet numbers to see if they are reachable from the host.  The first
one that is unreachable is used.  The subnet numbers are chosen from the
private network numbers specified by the Internet Engineering Task Force
(IETF) in RFC 1918 (http://www.isi.edu/in-notes/rfc1918.txt).

Remember that the host-only network that virtual machines reside on
will not be accessible outside the host.  This means that it is ok to
use the same number on different systems so long as you do not enable
communication between these networks.

EOF
}

sub promptForHostOnlyNetworkConfig {
    my $net = "";
    my $netmask = "";
    my $firstTime = 1;
    my $answer;

    while ($net eq "" || $netmask eq "") {
	#
	# First ask if probing should be tried.
	#
	do {
	    $answer = query("Probe for an unused network number? (yes/no/help)", "yes");
	    if ($answer =~ /^[Yy]/) {
		($net, $netmask) = probeForNetConfig();
		next;
	    }
	    printHostOnlyHelp() if $answer =~ /^[Hh]/;
	} until $answer =~ /^[Nn]/;

	#
	# No probing; ask if they want to hand config it.
	#
	do {
	    $answer = query("Specify a network number? (yes/no/help)", "yes");
	    if ($answer =~ /^[Yy]/) {
		($net, $netmask) = promptForNetConfig();
		next;
	    }
	    printHostOnlyHelp() if $answer =~ /^[Hh]/;
	} until $answer =~ /^[Nn]/;

	#
	# Give them some help (the first time through the loop)
	# and then give them a chance to abort configuring host-only
	# networking.
	#
	if ($firstTime != 0 || $answer =~ /^Hh/) {
	    $firstTime = 0;
	    printHostOnlyHelp();
	}

	$answer = query("Do you want to configure host-only networking? (yes/no/help)", "yes");
	if ($answer =~ /^[Nn]/) {
	    last;
	}
	printHostOnlyHelp() if $answer =~ /^[Hh]/;
    }
    return ($net, $netmask);
}

sub setupHostOnlyNetworkConfig {
    my $net = shift;
    my $netmask = shift;

    print "\nConfiguring vmnet1 to use network $net and mask $netmask.\n";

    if (not defined($ENV{VNET_HOSTONLY})) {
	# don't print for an upgrade, only a clean install
	printISCCopyrightInfo();
    }
    #
    # 1. Setup the DHCP daemon:
    #    write a config file for the DHCP daemon
    #    setup a null leases file
    # 2. Configure the vnet1 interface (done by boot-time script).
    # 3. Add a route for vnet1 since it doesn't get done 
    #    when the interface is marked up (done by boot-time script).
    # 4. Record the net and mask in the global config file
    #    so the boot-time script will start DHCPD (done below).
    #
    # XXX save existing .conf and .leases files
    print "Creating $vmware_etc/$vmnet1.conf for vmnet-dhcpd\n"; 
    writeDHCPConfigFile($net, $netmask);

    print "Creating empty leases file $vmware_etc/$vmnet1.leases for vmnet-dhcpd\n";
    # XXX deal with errors
    open(DHCPLEASES, ">$vmware_etc/$vmnet1.leases");
    close(DHCPLEASES);
    Log("file $vmware_etc/$vmnet1.leases\n");
    # XXX also include backup file dhcpd creates
    Log("file $vmware_etc/$vmnet1.leases~\n");

    #
    # If the host looks to have dhcpd setup then check if
    # it's configured to ignore our subnet.  If not warn
    # them that they should do this or they may encounter
    # problems.
    #
    if (-f '/etc/dhcpd.conf' && `grep -c '^[ ]*subnet '$net'.0' /etc/dhcpd.conf` == 0) {
        print <<"EOF";

You appear to have dhcpd configured for normal use.  Beware that
you should modify the /etc/dhcpd.conf file to define the subnet
created for VMware's host-only network; otherwise dhcpd will fail
to run if it is started with the $vmnet1 network interface enabled.
A possible subnet configuration to add to dhcpd.conf is:

subnet $net.0 netmask $netmask {
    # note that no range is given so dhcpd will not try to
    # assign IP addresses; this must be done by vmnet-dhcpd
}

If you do not want to edit dhcpd.conf then you need to always start
the program with an explicit list of your network interfaces (leaving
out $vmnet1); e.g.

dhcpd eth0

Consult the dhcpd(8) and dhcpd.conf(5) manual pages for details.

EOF
	query("Hit return to continue:", "");
    }

    #
    # If the host looks to have samba setup then check if
    # it's configured to respond on our subnet.  If not then
    # warn them that they must list the interface explicitly
    # if they want to server VMs on the host-only subnet.
    #
    if (-f '/etc/smb.conf' && `grep -c '^[ ]*interfaces[ ]*=.*'$net /etc/smb.conf` == 0 ) {
	my $interfaces = printSmbInterfaces;
        print <<"EOF";

You appear to have Samba configured for normal use.  Note that
if you want to offer service to virtual machines running on the
host-only network that you must update your /etc/smb.conf file
to list the network interfaces on your system by adding the line:

interfaces = $interfaces $net.1/24

You may also need to update any related security controls you
might have setup such as the "hosts allow" specification.

Consult the smb.conf(5) manual page for more details.

EOF
	query("Hit return to continue:", "");
    }
    print "\n";
}

sub printISCCopyrightInfo() {
  system((defined($ENV{PAGER}) ? $ENV{PAGER} : "/bin/more") . " DHCP-COPYRIGHT");
  print "\n";
  query("Hit return to continue:", "");
  print "\n";
}


sub configureHostOnlyNetworking {
    my $net = "";
    my $netmask = "";
    my $answer;

    if (not defined($ENV{VNET_HOSTONLY})) {
	print "\n";
	do {
	    $answer = query("Configure host-only networking support? (yes/no/help)", "yes");
	    printHostOnlyHelp() if !($answer =~ /^[YyNn]/);
	} until $answer =~ /^[YyNn]/;
	if ($answer =~ /^[Yy]/) {
	    ($net, $netmask) = promptForHostOnlyNetworkConfig();
	    # check if user aborted at lower-level prompt
	    $answer = "no" if ($net eq "" and $netmask eq "");
	}
    } else {
	$answer = $ENV{VNET_HOSTONLY};
	$net = $ENV{VNET_HOSTONLY_NETWORK};
	$netmask = $ENV{VNET_HOSTONLY_NETMASK};
    }
    setupHostOnlyNetworkConfig($net, $netmask) if ($answer =~ /^[Yy]/);
    Log("answer VNET_HOSTONLY $answer\n");
    return ($net, $netmask);
}

sub Host_Install {
  my $option = shift;
  my $networking;
  my $answer;
  my ($driver_src, $driver_path, $vnet_src, $vnet_path);
  my $vnet;
  my $hostOnlyNetwork;
  my $hostOnlyNetMask;
  my $i;

  checkHostPlatform();
  
  message("Installing host-specific files.\n");
  
  
  print "Finding an installable vmmon module\n"; 
  ($driver_src, $driver_path) = Find_Module("driver",$option);
  ($driver = $driver_src) =~ s/\./_/g;
  
  
  &Install(0444, "$driver_path/$driver_src", 
           "$modulesdir/preferred/misc/vmmon") 
    if (-d "$modulesdir/preferred");
  &Install(0444, "$driver_path/$driver_src", 
           "$modulesdir/$version/misc/vmmon") 
    if (-d "$modulesdir/$version");
  
  if (not $ENV{NETWORKING}) {
    if ($option eq "default") { 
      $answer = "yes";
    } else {
      $answer = query("Do you want to install vmnet (networking " .
                      "support for the VM)?", "yes");
    }
    $networking = ($answer =~ /^[yY]/);
    Log("answer NETWORKING $answer\n");
  } else {
    Log("answer NETWORKING $ENV{NETWORKING}\n");
    $networking = ($ENV{NETWORKING} =~ /^[yY]/);
  }
  
  
  
  if ($networking) {
    message("Installing networking support.\n");
    print "\nFinding an installable vmnet module\n";
    ($vnet_src, $vnet_path) = Find_Module("vmnet",$option);
    ($vnet = $vnet_src) =~ s/\./_/g;
    
    &Install(0444, "$vnet_path/$vnet_src", 
             "$modulesdir/preferred/misc/vmnet") 
      if (-d "$modulesdir/preferred");
    &Install(0444, "$vnet_path/$vnet_src", 
             "$modulesdir/$version/misc/vmnet") 
      if (-d "$modulesdir/$version");
  } else {
    message("No networking support will be installed.\n");
  }
  
  message("/dev/vmmon\n");
  system("mknod /dev/vmmon c 10 165") and die "mknod /dev/vmmon: $!.\n";
  chmod 0600, "/dev/vmmon" or die "\n### chmod /dev/vmmon: $!.\n";
  Log("file /dev/vmmon\n");
  
  if ($networking) {
    message("/dev/vmnet*\n");
    $i = 0;
    foreach $vnet ("/dev/vmnet0", "/dev/vmnet1", "/dev/vmnet2", "/dev/vmnet3") { 
      message("mknod $vnet\n");
      system("mknod $vnet c 119 $i") and die "mknod: $!.\n";
      chmod 0600, $vnet or die "chmod: $!.\n"; 
      Log("file $vnet\n");
      $i++;
    }
    
    message("$initdir/init.d/vmware\n");
    &Install(0555, "load-vmx86.dist", "$initdir/init.d/vmware");
    
    ($hostOnlyNetwork, $hostOnlyNetMask) = configureHostOnlyNetworking();
    
  } else {
    
    message("$initdir/init.d/vmware\n");
    &Install(0555, "load-vmx86-novmnet.dist", "$initdir/init.d/vmware");
    
    $hostOnlyNetwork = "";
    $hostOnlyNetMask = "";
    
  }
  
  symlink "../init.d/vmware", "$initdir/rc3.d/S99vmware";
  symlink "../init.d/vmware", "$initdir/rc5.d/S99vmware";
  symlink "../init.d/vmware", "$initdir/rc6.d/K08vmware";
  
  
  ###
  ### write the config file
  ###
  open(CONFIGFILE,">$vmware_etc/config");
  print CONFIGFILE "vmware.fullpath = $bindir/vmware\n";
  print CONFIGFILE "wizard.fullpath = $bindir/vmware-wizard\n";
  
  if (defined($hostOnlyNetwork) &&  $hostOnlyNetwork ne "") { 
    print CONFIGFILE "vmnet.HostOnlyNetwork = $hostOnlyNetwork\n";
    print CONFIGFILE "vmnet.HostOnlyNetMask = $hostOnlyNetMask\n";
  }
  close(CONFIGFILE);
  
  Log("file $vmware_etc/config\n");
  
  
  # Avoid reboot.
  message("Running $initdir/init.d/vmware start\n");
  system("$initdir/init.d/vmware start") and die "\n### vmware start: $!.\n";
  print "Installation successful \n";
  Log("Installation completed\n");
}


sub Legacy_Uninstall_vmx86 { 
  my @vmx86files = ("/dev/vmx86", "/dev/vnet0", "/dev/vnet1",  "/dev/vnet2",  "/dev/vnet3", 
                    "$initdir/init.d/vmx86","$initdir/rc3.d/S99vmx86",
                    "$initdir/rc5.d/S99vmx86","$initdir/rc6.d/K08vmx86");
 
  my @vmmonfiles = ("/dev/vmmon", 
                    "/dev/vmnet0",  "/dev/vmnet1",  "/dev/vmnet2",  "/dev/vmnet3", 
                    "$initdir/init.d/vmware",
                    "$initdir/rc3.d/S99vmware",
                    "$initdir/rc5.d/S99vmware",
                    "$initdir/rc6.d/K08vmware");
  
  my ($file,$oldlibdir);



  if (-e "/dev/vmx86") {
    ###
    ### host
    ###
    
    message("Running $initdir/init.d/vmx86 stop\n");
    system("$initdir/init.d/vmx86 stop") if -x "$initdir/init.d/vmx86";
   

    foreach $file (@vmx86files) { 
      message("removing legacy $file\n");
      unlink $file;
    }
    
    message ("removing legacy modules \n");
    unlink <$modulesdir/preferred/misc/vmx86-*>;
    unlink <$modulesdir/preferred/misc/vnet-*>;
    unlink <$modulesdir/$version/misc/vmx86-*>;
    unlink <$modulesdir/$version/misc/vnet-*>;
    
    #### 
    #### site
    ####
    
    ($oldlibdir = $legacylibdir) =~ s/vmware/vmx86/;
    if (-d "$oldlibdir") {
      my @oldfiles = ("vmm","BIOS.ROM","nvram","config", "demo", "vmx86-test");
      foreach $file (@oldfiles) {
        message("$oldlibdir/$file\n");
        unlink "$oldlibdir/$file" or warn "unlink $oldlibdir/$file: $!.\n";
      }
      rmdir "$oldlibdir" or warn "Could not remove $oldlibdir: $!.\n";
    }
    
    unlink "$bindir/vmx86" or warn "unlink $bindir/vmx86: $!.\n";
    message("$mandir/man1/vmx86.1 (Manual page)\n");
    unlink "$mandir/man1/vmx86.1" or warn "unlink $mandir/man1/vmx86.1: $!.\n";
    
  }
  
  if (-e "/dev/vmmon") {
    
    message("Running $initdir/init.d/vmware stop\n");
    system("$initdir/init.d/vmware stop") if -x "$initdir/init.d/vmware";
    
    foreach $file (@vmmonfiles) { 
      message("removing legacy $file\n");
      unlink $file;
    }
    
    message ("removing legacy modules \n");
    unlink <$modulesdir/preferred/misc/vmmon>;
    unlink <$modulesdir/preferred/misc/vmnet>;
    unlink <$modulesdir/$version/misc/vmmon>;
    unlink <$modulesdir/$version/misc/vmnet>;
    
    foreach $file ("vmware", "vmnet-bridge", "vmnet-sniffer") {
      message("$bindir/$file\n");
      unlink "$bindir/$file" or warn "unlink $bindir/$file: $!.\n";
    }
    
    
    foreach $file (@libfiles) {
      message("$legacylibdir/$file\n");
      unlink "$legacylibdir/$file" or warn "unlink $legacylibdir/$file: $!.\n";
    }
    
    message("$legacylibdir/nvram (Default NVRAM image)\n");
    unlink "$legacylibdir/nvram" or warn "unlink $legacylibdir/nvram: $!.\n";
    
    message("$legacylibdir/config (Default site configuration)\n");
    unlink "$legacylibdir/config" or warn "unlink $legacylibdir/config: $!.\n";
    
    
    message("$mandir/man1/vmware.1 (Manual page)\n");
    unlink "$mandir/man1/vmware.1" or warn "unlink $mandir/man1/vmware.1: $!.\n";
    
    
  }
}

sub Legacy_Uninstall { 

  my $answer = query("I've detected host files from early pre-releases. Delete them?","yes");

  if ($answer =~/^[Yy]/) { 
  } else {
    die "Bye!\n";
  }
 
  print <<"EOT";

Previous versions of the installation script did not record the
location of the installed files.  

If you installed pre-release versions of vmx86 or vmware in
non-standard locations, you will need to remove these files by hand.

This legacy uninstall script will only run once. If it fails during 
this phase, legacy files will remain 

We apologize for the inconvenience.

Thank you. 


EOT

  
  
  Init("legacy");
  Legacy_Uninstall_vmx86();
  
}


sub ReadLocations { 
  open(LOCATIONS,$vmware_locations);
  while (<LOCATIONS>) { 
    if (/build (\S+)/) {
      print "VMware for Linux was installed,  version = $1 \n";
    } elsif (/answer (\S+) (\S+)/) {
      $previousAnswer{$1} = $2;
    }
  }
}

sub Uninstall { 
  my ($dir, %dirs);
  
  
  message("Uninstall()\n");


  open(LOCATIONS,$vmware_locations);
  while (<LOCATIONS>) { 
    if (/build (\S+)/) {
      print "Uninstalling VMware  version = $1 \n";
    } elsif (/file (\S+)/) { 
      message("Unlinking $1\n");
      unlink $1 or warn "unlink $1:$!.\n";
    } elsif (/directory (\S+)/) {
      $dirs{$1} = $1;
    }
  }
      
  close(LOCATIONS); 
  unlink $vmware_locations or unlink "rmdir $vmware_locations: $!.\n";
  
  foreach $dir (keys %dirs) { 
    message("Unlinking directory $dir\n");
    rmdir $dir or warn "rmdir $dir:$!.\n";
  } 
}

  

sub Site_Install {
  my $option = shift;
  my $file;
  my $shared = 0;
  my $answer;
  my %allfiles;
  my %basename;
  my $base;

  message("Installing site-specific files.\n");
  
  
  foreach $file (@programs) { 
    if ($file eq "vmware") { 
      $allfiles{"$bindir/$file"} = 04555;
    } else {
      $allfiles{"$bindir/$file"} = 0555;
    }
    $basename{"$bindir/$file"} = $file;
  }
 
  $allfiles{"$mandir/man1/vmware.1"} = 0444;
  $basename{"$mandir/man1/vmware.1"} = "vmx86.1.dist";
  
  
  foreach $file (keys %allfiles) {
    message("$file\n");
    $shared = 0;
    if (-e "$file") { 
      $base = $basename{"$file"};
      if (defined($previousAnswer{"SHARED_$base"})) { 
        if ($previousAnswer{"SHARED_$base"} eq "$file") { 
          $shared = 1;
        }
      } elsif ($option eq "default") { 
        $shared = 1;
      } else {
        $answer = query("File $file already exists. Use it? (yes/no)","yes");
        if ($answer =~/^[Yy]/) {
          $shared = 1;
        }
      }
    }
    
    if ($shared) { 
      Log("answer SHARED_$base $file\n");
    } else { 
      print "Installing $file\n";
      &Install($allfiles{$file}, $basename{$file}, $file);
    }
  }
  
}




sub Cleanup { 
  if ($createdBuildDir ne "") { 
    message("Removing $createdBuildDir\n");
    system("rm -rf $createdBuildDir");
  }
}

################################################################
#                        Main                                  #
################################################################

sub main {

 
    my ($option, $recursive) = @_;
    my $answer;
    my $initscript;
    
    ###
    ### legacy deinstall (previous releases)
    ###
    
    if (-d $vmware_tmpdir)  {
      print "VMware for Linux \n\n" . 
        "The previous installation of VMware was incomplete, or at least the \n" .
          "directory /tmp/vmware-tmp exists.\n\n";
      
      
      $answer = query("Would you like me to 'rm -rf' /tmp/vmware-tmp? (yes/no)","no");
      if ($answer =~/^[Yy]/) { 
        system("rm -rf /tmp/vmware-tmp");
      } else { 
        die "Please remove $vmware_tmpdir manually\n";
      }
    }
    
    if (!(-d $vmware_etc) && (-e "/dev/vmx86" )) {
      Legacy_Uninstall();
      print <<"LEGACY";

The uninstallation of the pre-beta software was successful. 
Please restart ./install.pl to install the new one.

LEGACY
      die "Thank you\n";
    }
    
  
     

    if (! $recursive && $option ne "default" && $option ne "-default") {
	print << "EOM";


--------------------------------------------
         VMware for Linux  installer 

    Copyright (C) 1998,1999 VMware, Inc.
--------------------------------------------

EOM
    }

    if ($option eq "uninstall" ||
        $option eq "upgrade") { 
      if (!(-e $vmware_locations)) { 
        die "VMware is currently not installed. Cannot upgrade or uninstall \n";
      }
      
      ReadLocations();

    } else { 
      
      ###
      ### Upgrade/Deinstall
      ###
      if (!$recursive && -e $vmware_locations) {
        
        ReadLocations();

        print "An existing distribution of VMware for Linux has been detected.\n";
        print "Location of existing VMware binaries: $previousAnswer{BINDIR}\n";
        if (defined($previousAnswer{NETWORKING}) && 
            $previousAnswer{NETWORKING} =~ /^[yY]/) { 
          print "Existing installation has networking enabled\n";
        } else { 
          print "Existing installation has networking disabled\n";
        }
        
        if (defined($previousAnswer{VNET_HOSTONLY}) && 
            $previousAnswer{VNET_HOSTONLY} =~ /^[yY]/) { 
          print "Existing installation has host-only networking enabled\n";
        } else { 
          print "Existing installation has host-only networking disabled\n";
        }
        print "\n";
        
        $answer = query("Would you like to upgrade VMware for Linux? (yes/no)","yes");
        if ($answer =~/^[Yy]/) { 
          $option = "upgrade";
        } else { 
          $answer = query("Would you like to uninstall VMware for Linux ? (yes/no)","no");
          if ($answer =~/^[Yy]/) { 
            $option = "uninstall";
          } else { 
            die "You need to choose between upgrading and uninstalling\n";
          }
        }
        
      }
    }
    
    print "\n";
    if (($option eq "upgrade" || $option eq "uninstall") &&
        -e $vmware_locations && 
        defined($previousAnswer{INITDIR})) {
      
      
      
      $initscript = "$previousAnswer{INITDIR}/init.d/vmware";
      message("Running $initscript stop\n");
      if (-x "$initscript") { 
        if (system("$initscript stop")) { 
          die "$initscript stop: failed -- are you running a virtual machine?\n";
        }
      }

      open(LSMOD,"/sbin/lsmod | egrep '(vmmon|vmnet)'|");
      while (<LSMOD>) {
        print "Module is still loaded: $_";
        die "Cannot upgrade or uninstall. Are you running a virtual machine?\n";
      }
      close(LSMOD);
      
      Uninstall();
      
    }
    
    if ($recursive) { 
      return;
    }
    
    if ($option eq "uninstall") {
      die "Thank you for trying VMware. VMware is now uninstalled\n";
    }
    
    
    ###
    ###
    ### 
    
    print "\n";
    while ($option eq "none") {
      $answer = query("Perform default installation? (yes/no/help)", "yes");
      if ($answer =~ /^[Yy]/) {
        $option = "default";
      } elsif ($answer =~ /^[Nn]/) {
        $option = "all";
      } else {
        print << "EOM";

If you answer YES, this script will install the executables
in $default_bindir, the manual pages in $default_mandir, 
and the boot-time init scripts in $default_initdir.  
If these directories contain a previous installation, it will be 
uninstalled first. If these directories do not exist, they will be 
created. No questions will be asked, unless problems arise.

If you answer NO, this script will prompt for the names
of the installation directories (unless BINDIR, MANDIR, and INITDIR are set), 
and may ask further questions to direct the installation.

Please answer yes or no, or type ^C to exit.

EOM
       }
    }

    if ($option =~ /^-/) {
      $option = substr($option, 1);
    }
    
    if ($option eq "default") {
      $ENV{BINDIR} = $default_bindir;
      $ENV{MANDIR} = $default_mandir;
      $ENV{INITDIR} = $default_initdir;
    } 

    if ($option eq "upgrade") { 
      foreach $answer ("BINDIR","MANDIR","INITDIR","NETWORKING",
                       "BUILDR_driver","BUILDR_vnet", "VNET_HOSTONLY", 
                       "VNET_HOSTONLY_NETWORK", "VNET_HOSTONLY_NETMASK") { 
        if (defined($previousAnswer{$answer})) { 
          message("Upgrading $answer with $previousAnswer{$answer}\n");
          $ENV{$answer} = $previousAnswer{$answer};
        }
      }
    }
    
    
    if ($option eq "default" || 
        $option eq "all" || 
        $option eq "upgrade") { 
      &Show_EULA();
      OpenLog();
      &Init($option);
      &Site_Install($option);
      &Host_Install($option);
      close (LOCATIONS);
      &Cleanup();
      print <<"DONE";

To run vmware, exit from root, leave this distribution directory
and type '$bindir/vmware'.  Remember to visit http://www.vmware.com
to get software updates. 

Enjoy,
   
    --the VMware team
DONE

    } else {
      
	die << "EOM"
$0: unknown argument $option.

Usage: $0 [MODE]
where MODE is one of:

    all             - install everything
    default         - quietly install everything in the default locations
    upgrade         - upgrade to a new version of VMware
    uninstall       - remove VMware all its associated files
EOM
   }
}


###############
#             #
# Really Main #
#             #
###############


die "You must be root to install the VMware software.\n" unless $> == 0;

$umask = umask 022;

$smp = (`uname -v` =~ / SMP /) ? 1 : 0;
chop($version = `uname -r`);

$option = shift;

if (defined($option) && $option eq "-v") {
  $verbose = 1;
  $option = shift;
}

$option = "none" unless defined($option);

main($option, 0);

cleanup:
umask $umask;

