#!/usr/bin/perl -w
#
# Copyright (C) 1998, 1999 VMware, Inc.  All Rights Reserved.
#
# Wizard to generate a 'safe rawdisk access file'
#
# If your copy of perl is not in /usr/bin, please adjust the line above.


$device = shift;

if (!defined($device)) { 
  die "Usage: make-saferawdisk (/dev/hda|/dev/hdb|/dev/hdc|/dev/hdd) \n";
}

$ok = "";
for $controller ('a','b','c','d') {
  if ($device eq "/dev/hd".$controller) {
    $ok = $controller;
  }
}

if ($ok eq "") { 
  die "Usage: make-saferawdisk (/dev/hda|/dev/hdb|/dev/hdc|/dev/hdd) \n";
}

$op = "/sbin/fdisk -lu $device";

print "executing '$op'\n\n";

sub queryString { 
  my ($message, $defaultreply) = @_;
  my $reply;
  my $optSpace = substr($message, -1) eq "\n" ? "" : " ";
  print $message . $optSpace . "[" . $defaultreply . "] ";
  
  chop($reply = <STDIN>);
  $reply =~ s/^\s*//;			# Kill leading whitespace
  $reply =~ s/\s*$//;			# Kill trailing whitespace
  $reply = $defaultreply if $reply eq "";
  return $reply;
}

sub queryAccess { 
  
 RETRY: while(1) { 
    my ($message, $defaultreply) = @_;
    my $reply;
    my $optSpace = substr($message, -1) eq "\n" ? "" : " ";
    print $message . $optSpace . "[" . $defaultreply . "] ";
	   
    chop($reply = <STDIN>);
    $reply =~ s/^\s*//;			# Kill leading whitespace
    $reply =~ s/\s*$//;			# Kill trailing whitespace
    $reply = $defaultreply if $reply eq "";
    if ($reply =~ /^[Yy]$|^[Yy][Ee][Ss]$/) {
      return "yes";
    }
    if ($reply =~ /^[Nn]$|^[Nn][Oo]$/) {
      return "no";
    }
    if ($reply =~ /^[Rr]$|^[Rr][Dd][Oo][Nn][Ll][Yy]/) { 
      return "rdonly";
    }
  }
}


open(OUTPUT,"$op|");

while (<OUTPUT>) { 
  print "FDISK: $_";
  @arr = split;
  if (defined($arr[0]) &&
      $arr[0] =~/$device(\S)/) {
    if ($arr[1] eq "*") { 
      $start{$arr[0]} = $arr[2];
      $end{$arr[0]} = $arr[3];
    } else {
      $start{$arr[0]} = $arr[1];
      $end{$arr[0]} = $arr[2];
    }
  }
}

print "\n";


@sorted = sort (keys %start);


foreach $i (@sorted) { 
  printf("PARTITION %s sectors %10d %10d\n",
         $i, $start{$i}, $end{$i});
}

print "\n";


###
### ask questions for each partition
###


foreach $i (@sorted) {
  $access{$i} = queryAccess("Allow access to partition $i (yes/no/rdonly)","no");
}

$min = 0;
foreach $i (@sorted) { 
  if ($min==0 || 
      $min >= $start{$i}) {
    $min = $start{$i} -1 ;
  }
}

if ($min>0) { 
  $mbr = queryAccess("Allow access below partition 0 (first $min sectors) (yes/no/rdonly","rdonly");
}


$fileName = queryString("Enter filename for safe raw-device :","safe-hd" .$ok);

open(OUTPUT,">$fileName");

print OUTPUT "DEVICE $device\n\n";


if ($mbr eq "rdonly") {
  print OUTPUT "### Allow read only access to MBR\n";
  print OUTPUT "RDONLY 0 $min\n\n";
} elsif ($mbr eq "yes") { 
  print OUTPUT "### Allow read/write access to MBR\n";
  print OUTPUT "ACCESS 0 $min\n\n";
}


print "### Partition table ###\n";

foreach $i (@sorted) { 
  printf OUTPUT ("### %s start=%10d end=%10d %s\n",
                 $i, $start{$i}, $end{$i},
                 ($access{$i} ? "ALLOW" : "DENY"));
}
print "\n";
foreach $i (@sorted) { 
  if ($access{$i} eq "yes") { 
    printf OUTPUT ("ACCESS %10d %10d \n",
                   $start{$i}, $end{$i});
  } elsif ($access{$i} eq "rdonly") { 
    printf OUTPUT ("RDONLY %10d %10d\n", $start{$i}, $end{$i});
  }
}


close(OUTPUT);

print "File '$fileName' generated:\n";
system("cat $fileName");







  
  
